﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace IEnumerableTest
{
    public class People
    {
        public int ID { get; set; }
        public string Name { get; set; }
        public int Age { get; set; }

        public override string ToString()
        {
            return string.Format("ID:{0},Name{1},Age{2}", ID, Name, Age);
        }
    }

    public class PeopleList
    {
        private readonly List<People> _people;
        public int OldAge = 31;

        public PeopleList()
        {
            _people = new List<People>();
            for (int i = 0; i < 5; i++)
            {
                _people.Add(new People {Name = "P" + i, ID = i, Age = 30 + i});
            }
        }

        public IEnumerable<People> PeopleEnumerable1
        {
            get { return _people; }
        }

        public IEnumerable<People> PeopleEnumerable2
        {
            get
            {
                foreach (People people in _people)
                {
                    yield return people;
                }
            }
        }


        public IEnumerable<People> PeopleEnumerable3
        {
            get
            {
                foreach (People people in _people)
                {
                    yield return people;
                }
            }
        }

        public IEnumerable<People> OlderPeoples
        {
            get
            {
                foreach (People people in _people)
                {
                    if (people.Age > OldAge)
                        yield return people;
                }
            }
        }
    }

    internal class Program
    {
        private static void Main(string[] args)
        {
            var peopleList = new PeopleList(); //内部构造函数插入了五个元素
            IEnumerator<People> e1 = peopleList.PeopleEnumerable3.GetEnumerator();
            if (e1.Current == null)
            {
                Console.WriteLine("迭代器生成后Current为空");
            }
            int i = 0;
            while (i < 8) //总共只有五个元素，看看一直迭代会发生什么效果
            {
                e1.MoveNext();
                if (e1.Current == null)
                {
                    Console.WriteLine("迭代第{0}次后为空", i);
                }
                else
                {
                    Console.WriteLine("迭代第{0}次后为{1}", i, e1.Current.Name);
                }
                i++;
            }

            Console.WriteLine("不修改OldAge参数");
            foreach (People olderPeople in peopleList.OlderPeoples)
            {
                Console.WriteLine(olderPeople);
            }

            Console.WriteLine("修改了OldAge参数");
            i = 0;
            foreach (People olderPeople in peopleList.OlderPeoples)
            {
                Console.WriteLine(olderPeople);
                i++;
                if (i > 0)
                    peopleList.OldAge = 33;
            }

            int age = 31;
            Console.WriteLine("在迭代中修改变量值");
            i = 0;
            IEnumerable<People> e4 = peopleList.PeopleEnumerable1.Where(d => d.Age > age);
            IEnumerator<People> e5 = e4.GetEnumerator();
            i = 0;
            while (e5.MoveNext())
            {
                Console.WriteLine(e5.Current);
                i++;
                if (i > 0)
                    age = 33;
            }
            age = 31;
            Console.WriteLine("在Lambda表达式中修改变量值");
            foreach (People b in e4)
            {
                Console.WriteLine(b);
                i++;
                if (i > 0)
                    age = 33;
            }
            Console.WriteLine("Skip的元素是否会被访问到？");
            IEnumerable<People> e6 = peopleList.PeopleEnumerable1.Select(d =>
            {
                Console.WriteLine(d);
                return d;
            }).Skip(3);
              Console.WriteLine("只枚举，什么都不做：");
            foreach (var  r in e6){}  

             Console.WriteLine("转换为实体集合，再次枚举");
            IEnumerable<People> e7 = e6.ToList();
            foreach (var r in e7){}



            List<People> peoples=new List<People>(){new People(){Name = "PA"}};
            Console.WriteLine("将一个虚拟枚举A连接到集合B，并在枚举A阶段修改集合B的元素");
            var e8 = peopleList.PeopleEnumerable1.Concat(peoples);
            i = 0;
            foreach (var people in e8)
            {
                Console.WriteLine(people);
                i++;
                if (i > 0)
                  peoples.Add(new People(){Name = "PB"});
            }

        }
    }
}